
;**************************** Allgemeine Routinen **************************

;***************************************************************************
;  Rom nach Ram kopieren Z - Rom-Adresse, Y- Ram-Adresse, immer 20 Byte
;
;  Scratch-Reg: ZH, ZL, YH, YL
;***************************************************************************

rom_ram:
		push	TEMP_A
		push	TEMP_B

        ldi     TEMP_A,20

r00:
        lpm		TEMP_B,Z+
        st      Y+,TEMP_B
        dec     TEMP_A
        brne    r00

		pop		TEMP_B
		pop		TEMP_A

        ret

;*********************************************************************
;  Warte 5ms
;
;  Scratch-Reg: -
;*********************************************************************

wait_5ms:
        push    ZL
        push    ZH

        load_p  Z,5000*(CYCLES_PER_US/4)
		        ; 5 mS warten, 4 = Anzahl Befehls-Zyclen vom sbiw + brcc

d_loop:
        sbiw    ZL,1
        brne    d_loop

        pop     ZH
        pop     ZL

        ret

;*********************************************************************
;  Write Display
;  Input:  TEMP_A - Daten
;  Scratch-Reg: TEMP_A
;*********************************************************************
write_led:
		push	TEMP_B
		ldi		TEMP_B,8				; Bitzhler
		cbi		LED_PORT,LED_CLOCK		; Clock L
		cbi		LED_PORT,LED_LOAD		; Load L	

write_led_loop:
		lsr		TEMP_A					; Bit0 in Carry
		brcs	write_led_high

		cbi		LED_PORT,LED_DATA
		rjmp	write_led_bit_ok

write_led_high:
		sbi		LED_PORT,LED_DATA

write_led_bit_ok:
		sbi		LED_PORT,LED_CLOCK
		nop
		cbi		LED_PORT,LED_CLOCK

		dec		TEMP_B
		brne	write_led_loop

		sbi		LED_PORT,LED_LOAD		; Load H
		pop		TEMP_B

		ret

;*********************************************************************
;  Write Display
;  Input:  TEMP_A - Daten  
;  Scratch-Reg: -
;*********************************************************************
ascii2led:
				push	ZH
				push	ZL
				push	TEMP_B
				push	TEMP_A				; ASCII-Wert retten
				mov		TEMP_A,LED_POS		; Posotion holen
				ori		TEMP_A,0b10100000	; Digit-Selekt
				rcall	write_led
				pop		TEMP_A

				subi	TEMP_A,' '			; Controlcodes abziehen
				ldi		TEMP_B,8			; Anzahl Byte pro Zeichen
				mul		TEMP_A,TEMP_B		; Offset Zeichen zum Tabellenstart
				load_p	Z,(2*ascii_tab)		; Tabelle
				add		ZL,MUL_L			; Byteoffset dazu
				adc		ZH,MUL_H

				ldi		TEMP_B,7			; Anzahl Zeilen

ascii2led_loop:
	            lpm 	TEMP_A,Z+ 
				rcall	write_led
				dec		TEMP_B
				brne	ascii2led_loop

				inc		LED_POS
				andi	LED_POS,$07
				
				pop		TEMP_B
				pop		ZL
				pop		ZH
				ret
